/*!
 * Ghostery for Firefox
 * http://www.ghostery.com/
 *
 * Copyright 2013 EVIDON, Inc. All rights reserved.
 * See ghostery_eula.txt for license.
 */

var utils = require('./utils'),
	bugDb = require('./bugdb').load,
	lsoDb = require('./lsodb').load;

// TODO THIS HAS TO BE SUPER FAST
function isBug(src, tab_url) {
	var i,
		id,
		priorities = ['high', 'regular', 'low'],
		regexes,
		db = bugDb.db,
		q;

	if (!db.fullRegex.test(src)) {
		return false;
	}

	// strip out the querystring, including the ?, to reduce false positives
	q = src.indexOf('?');
	if (q >= 0) {
		src = src.slice(0, q);
	}

	for (i = 0; i < priorities.length; i++) {
		regexes = db.regexes[priorities[i]];

		for (id in regexes) {
			// note: no hasOwnProperty check since our code is sandboxed

			// TODO can still produce false positives (when something that
			// TODO matches a tracker is in the path somewhere, for example)
			if (regexes[id].test(src)) {
				if (db.firstPartyExceptions[id] &&
					utils.fuzzyUrlMatcher(tab_url, db.firstPartyExceptions[id])) {
					return false;
				}

				return id;
			}
		}
	}

	return false;
}

// TODO THIS HAS TO BE SUPER FAST
function isLso(src) {
	var i,
		id,
		priorities = ['high', 'regular', 'low'],
		regexes,
		db = lsoDb.db,
		q;

	if (!db.fullRegex.test(src)) {
		return false;
	}

	// strip out the querystring, including the ?, to reduce false positives
	q = src.indexOf('?');
	if (q >= 0) {
		src = src.slice(0, q);
	}

	for (i = 0; i < priorities.length; i++) {
		regexes = db.regexes[priorities[i]];

		for (id in regexes) {
			// note: no hasOwnProperty check since our code is sandboxed

			// TODO can still produce false positives (when something that
			// TODO matches a tracker is in the path somewhere, for example)
			if (regexes[id].test(src)) {
				return id;
			}
		}
	}

	return false;
}

exports.isBug = isBug;
exports.isLso = isLso;
